# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from __future__ import annotations

import threading

from abc import ABC, abstractmethod
from typing import Dict, List, Optional, Tuple, Union

from ...core import SDK

from ...core import create_basic_authentication
from ...core import create_pkce_verifier_and_challenge_s256
from ...core import get_access_token
from ...core import get_client_auth
from ...core import get_client_id
from ...core import remove_token
from ...core import set_token
from ...core import get_token_repository

from ...core import AccelByteSDK
from ...core import HttpResponse
from ...core import NoHttpBackoffPolicy, NoHttpRetryPolicy
from ...core import Operation
from ...core import Timer, TimerStatus
from ...core import TokenRepository

from ...api.iam import authorize_v3
from ...api.iam import platform_token_grant_v3
from ...api.iam import token_grant_v3
from ...api.iam import token_revocation_v3
from ...api.iam import user_authentication_v3

from ...api.iam import authorize_v3_async
from ...api.iam import platform_token_grant_v3_async
from ...api.iam import token_grant_v3_async
from ...api.iam import token_revocation_v3_async
from ...api.iam import user_authentication_v3_async

from ...api.iam.operations.o_auth2_0 import AuthorizeV3ResponseTypeEnum
from ...api.iam.operations.o_auth2_0 import TokenGrantV3GrantTypeEnum


# region constants

DEFAULT_AUTO_REFRESH: bool = False
DEFAULT_REFRESH_RATE: float = 0.8
DEFAULT_SCOPE: str = "commerce account social publishing analytics"

# endregion constants


# region sync


def login_client(
    client_id: Optional[str] = None,
    client_secret: Optional[str] = None,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    sdk = kwargs.get("sdk") or SDK
    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)

    if not client_id and client_secret:
        return None, HttpResponse.create_error(400, "Invalid client ID.")

    if not client_id:
        client_auth, error = get_client_auth(sdk=sdk)
        if error:
            return None, error
        client_id, config_client_secret = client_auth
        if not client_secret:
            client_secret = config_client_secret

    if not client_secret:
        sdk.logger.warning(
            "The use of a Public OAuth Client is highly discouraged for this use case."
        )

    x_additional_headers = x_additional_headers or {}
    x_additional_headers["Authorization"] = create_basic_authentication(
        client_id, client_secret
    )

    token: Optional[str] = None
    if refresh_if_possible:
        token = try_refresh_login(x_additional_headers=x_additional_headers, **kwargs)

    if not token:
        token, error = token_grant_v3(
            grant_type=TokenGrantV3GrantTypeEnum.CLIENT_CREDENTIALS,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        _, error = set_token(token=token, sdk=sdk)
        if error:
            return None, error

    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginClientTimer(
                    0,
                    client_id=client_id,
                    client_secret=client_secret,
                    x_additional_headers=x_additional_headers,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    assert token, "Token value missing!"
    return token, None


def login_platform(
    platform_id: str,
    platform_token: str,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)

    token: Optional[str] = None
    if refresh_if_possible:
        token = try_refresh_login(x_additional_headers=x_additional_headers, **kwargs)

    if not token:
        token, error = platform_token_grant_v3(
            platform_id=platform_id,
            platform_token=platform_token,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        _, error = set_token(token=token, sdk=kwargs.get("sdk"))
        if error:
            return None, error

    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginPlatformTimer(
                    0,
                    platform_id=platform_id,
                    platform_token=platform_token,
                    x_additional_headers=x_additional_headers,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    assert token, "Token value missing!"
    return token, None


def login_user(
    username: str,
    password: str,
    scope: Optional[Union[str, List[str]]] = None,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)

    token: Optional[str] = None
    if refresh_if_possible:
        token = try_refresh_login(x_additional_headers=x_additional_headers, **kwargs)

    if not token:
        (
            code_verifier,
            code_challenge,
            code_challenge_method,
        ) = create_pkce_verifier_and_challenge_s256()

        if scope is None:
            scope = DEFAULT_SCOPE
        if isinstance(scope, list):
            scope = " ".join(scope)

        client_id, error = get_client_id(sdk=kwargs.get("sdk"))
        if error:
            return None, error

        request_id, error = authorize_v3(
            client_id=client_id,
            response_type=AuthorizeV3ResponseTypeEnum.CODE,
            code_challenge=code_challenge,
            code_challenge_method=code_challenge_method,
            scope=scope,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        code, error = user_authentication_v3(
            password=password,
            request_id=request_id,
            user_name=username,
            client_id=client_id,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        token, error = token_grant_v3(
            grant_type=TokenGrantV3GrantTypeEnum.AUTHORIZATION_CODE,
            code=code,
            code_verifier=code_verifier,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        _, error = set_token(token=token, sdk=kwargs.get("sdk"))
        if error:
            return None, error

    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginUserTimer(
                    0,
                    username=username,
                    password=password,
                    scope=scope,
                    x_additional_headers=x_additional_headers,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    assert token, "Token value missing!"
    return token, None


def logout(x_additional_headers: Optional[Dict[str, str]] = None, **kwargs):
    access_token, error = get_access_token(sdk=kwargs.get("sdk"))
    if error:
        return None, error

    client_auth, error = get_client_auth(sdk=kwargs.get("sdk"))
    if error:
        return None, error

    basic_auth = create_basic_authentication(*client_auth)
    x_additional_headers = x_additional_headers or {}
    x_additional_headers["Authorization"] = basic_auth

    token_refresher = unset_on_demand_token_refresher(reset=False, **kwargs)

    _, error = token_revocation_v3(
        token=access_token, x_additional_headers=x_additional_headers, **kwargs
    )
    if error:
        if token_refresher is not None:
            set_on_demand_token_refresher(token_refresher=token_refresher, **kwargs)
        return None, error

    _, error = remove_token(sdk=kwargs.get("sdk"))
    if error:
        return None, error

    if token_refresher is not None:
        token_refresher.reset()

    return None, None


def refresh_login(
    refresh_token, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    token, error = token_grant_v3(
        grant_type=TokenGrantV3GrantTypeEnum.REFRESH_TOKEN,
        refresh_token=refresh_token,
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    _, error = set_token(token=token, sdk=kwargs.get("sdk"))
    if error:
        return None, error

    return token, None


def try_refresh_login(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
) -> Optional[str]:
    token_repo = get_token_repository(raise_when_none=False, sdk=kwargs.get("sdk"))
    if token_repo is None:
        return None

    if token_repo.has_token_expired():
        refresh_token = token_repo.get_refresh_token()
        if not refresh_token:
            return None

        token, error = refresh_login(
            refresh_token=refresh_token,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error or not token:
            return None
    else:
        token = token_repo.get_token()

    return token


login = login_user


# endregion sync


# region async


async def login_client_async(
    client_id: Optional[str] = None,
    client_secret: Optional[str] = None,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    sdk = kwargs.get("sdk") or SDK
    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)

    if not client_id and client_secret:
        return None, HttpResponse.create_error(400, "Invalid client ID.")

    if not client_id:
        client_auth, error = get_client_auth(sdk=sdk)
        if error:
            return None, error
        client_id, config_client_secret = client_auth
        if not client_secret:
            client_secret = config_client_secret

    if not client_secret:
        sdk.logger.warning(
            "The use of a Public OAuth Client is highly discouraged for this use case."
        )

    x_additional_headers = x_additional_headers or {}
    x_additional_headers["Authorization"] = create_basic_authentication(
        client_id, client_secret
    )

    token: Optional[str] = None
    if refresh_if_possible:
        token = await try_refresh_login_async(
            x_additional_headers=x_additional_headers, **kwargs
        )

    if not token:
        token, error = await token_grant_v3_async(
            grant_type=TokenGrantV3GrantTypeEnum.CLIENT_CREDENTIALS,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        _, error = set_token(token=token, sdk=sdk)
        if error:
            return None, error

    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginClientTimer(
                    0,
                    client_id=client_id,
                    client_secret=client_secret,
                    x_additional_headers=x_additional_headers,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    assert token, "Token value missing!"
    return token, None


async def login_platform_async(
    platform_id: str,
    platform_token: str,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)

    token: Optional[str] = None
    if refresh_if_possible:
        token = await try_refresh_login_async(
            x_additional_headers=x_additional_headers, **kwargs
        )

    if not token:
        token, error = await platform_token_grant_v3_async(
            platform_id=platform_id,
            platform_token=platform_token,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        _, error = set_token(token=token, sdk=kwargs.get("sdk"))
        if error:
            return None, error

    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginPlatformTimer(
                    0,
                    platform_id=platform_id,
                    platform_token=platform_token,
                    x_additional_headers=x_additional_headers,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    assert token, "Token value missing!"
    return token, None


async def login_user_async(
    username: str,
    password: str,
    scope: Optional[Union[str, List[str]]] = None,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)

    token: Optional[str] = None
    if refresh_if_possible:
        token = try_refresh_login(x_additional_headers=x_additional_headers, **kwargs)

    if not token:
        (
            code_verifier,
            code_challenge,
            code_challenge_method,
        ) = create_pkce_verifier_and_challenge_s256()

        if scope is None:
            scope = DEFAULT_SCOPE
        if isinstance(scope, list):
            scope = " ".join(scope)

        client_id, error = get_client_id(sdk=kwargs.get("sdk"))
        if error:
            return None, error

        request_id, error = await authorize_v3_async(
            client_id=client_id,
            response_type=AuthorizeV3ResponseTypeEnum.CODE,
            code_challenge=code_challenge,
            code_challenge_method=code_challenge_method,
            scope=scope,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        code, error = await user_authentication_v3_async(
            password=password,
            request_id=request_id,
            user_name=username,
            client_id=client_id,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        token, error = await token_grant_v3_async(
            grant_type=TokenGrantV3GrantTypeEnum.AUTHORIZATION_CODE,
            code=code,
            code_verifier=code_verifier,
            x_additional_headers=x_additional_headers,
            **kwargs,
        )
        if error:
            return None, error

        _, error = set_token(token=token, sdk=kwargs.get("sdk"))
        if error:
            return None, error

    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginUserTimer(
                    0,
                    username=username,
                    password=password,
                    scope=scope,
                    x_additional_headers=x_additional_headers,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    assert token, "Token value missing!"
    return token, None


async def logout_async(x_additional_headers: Optional[Dict[str, str]] = None, **kwargs):
    access_token, error = get_access_token(sdk=kwargs.get("sdk"))
    if error:
        return None, error

    client_auth, error = get_client_auth(sdk=kwargs.get("sdk"))
    if error:
        return None, error

    basic_auth = create_basic_authentication(*client_auth)
    x_additional_headers = x_additional_headers or {}
    x_additional_headers["Authorization"] = basic_auth

    token_refresher = unset_on_demand_token_refresher(reset=False, **kwargs)

    _, error = await token_revocation_v3_async(
        token=access_token, x_additional_headers=x_additional_headers, **kwargs
    )
    if error:
        if token_refresher is not None:
            set_on_demand_token_refresher(token_refresher=token_refresher, **kwargs)
        return None, error

    _, error = remove_token(sdk=kwargs.get("sdk"))
    if error:
        return None, error

    if token_refresher is not None:
        token_refresher.reset()

    return None, None


async def refresh_login_async(
    refresh_token, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    token, error = await token_grant_v3_async(
        grant_type=TokenGrantV3GrantTypeEnum.REFRESH_TOKEN,
        refresh_token=refresh_token,
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    _, error = set_token(token=token, sdk=kwargs.get("sdk"))
    if error:
        return None, error

    return token, None


async def try_refresh_login_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
) -> Optional[str]:
    token_repo = get_token_repository(raise_when_none=False, sdk=kwargs.get("sdk"))
    if token_repo is None:
        return None

    refresh_token = token_repo.get_refresh_token()
    if not refresh_token:
        return None

    token, error = await refresh_login_async(
        refresh_token=refresh_token, x_additional_headers=x_additional_headers, **kwargs
    )
    if error or not token:
        return None

    return token


login_async = login_user_async


# endregion async


# region repeating


TIMER_KWARGS = {
    "retry_policy": NoHttpRetryPolicy(),
    "backoff_policy": NoHttpBackoffPolicy(),
}


class LoginTimerBase(ABC, Timer):
    @abstractmethod
    def run(self):
        pass

    @abstractmethod
    async def run_async(self):
        pass


class LoginClientTimer(LoginTimerBase):
    def __init__(
        self,
        interval: Union[int, float],
        /,
        *,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        x_additional_headers: Optional[Dict[str, str]] = None,
        repeats: Optional[int] = None,
        autostart: bool = False,
        repeat_on_exception: bool = False,
        **kwargs,
    ) -> None:
        self.kwargs = {
            "client_id": client_id,
            "client_secret": client_secret,
            "x_additional_headers": x_additional_headers,
            **kwargs,
            **TIMER_KWARGS,
        }
        super().__init__(
            interval,
            self.run,
            daemon=True,
            repeats=repeats,
            autostart=autostart,
            repeat_on_exception=repeat_on_exception,
        )

    def run(self):
        return login_client(**self.kwargs)

    async def run_async(self):
        return await login_client_async(**self.kwargs)


class LoginPlatformTimer(LoginTimerBase):
    def __init__(
        self,
        interval: Union[int, float],
        /,
        *,
        platform_id: str,
        platform_token: str,
        x_additional_headers: Optional[Dict[str, str]] = None,
        repeats: Optional[int] = None,
        autostart: bool = False,
        repeat_on_exception: bool = False,
        **kwargs,
    ) -> None:
        self.kwargs = {
            "platform_id": platform_id,
            "platform_token": platform_token,
            "x_additional_headers": x_additional_headers,
            **kwargs,
            **TIMER_KWARGS,
        }
        super().__init__(
            interval,
            self.run,
            daemon=True,
            repeats=repeats,
            autostart=autostart,
            repeat_on_exception=repeat_on_exception,
        )

    def run(self):
        return login_platform(**self.kwargs)

    async def run_async(self):
        return await login_platform_async(**self.kwargs)


class LoginUserTimer(LoginTimerBase):
    def __init__(
        self,
        interval: Union[int, float],
        /,
        *,
        username: str,
        password: str,
        scope: Optional[Union[str, List[str]]] = None,
        x_additional_headers: Optional[Dict[str, str]] = None,
        repeats: Optional[int] = None,
        autostart: bool = False,
        repeat_on_exception: bool = False,
        **kwargs,
    ) -> None:
        self.kwargs = {
            "username": username,
            "password": password,
            "scope": scope,
            "x_additional_headers": x_additional_headers,
            **kwargs,
            **TIMER_KWARGS,
        }
        super().__init__(
            interval,
            self.run,
            daemon=True,
            repeats=repeats,
            autostart=autostart,
            repeat_on_exception=repeat_on_exception,
        )

    def run(self):
        return login_user(**self.kwargs)

    async def run_async(self):
        return await login_user_async(**self.kwargs)


class RefreshLoginTimer(LoginTimerBase):
    def __init__(
        self,
        interval: Union[int, float],
        /,
        *,
        refresh_token: str,
        x_additional_headers: Optional[Dict[str, str]] = None,
        repeats: Optional[int] = None,
        autostart: bool = False,
        repeat_on_exception: bool = False,
        **kwargs,
    ) -> None:
        self.kwargs = {
            "refresh_token": refresh_token,
            "x_additional_headers": x_additional_headers,
            **kwargs,
            **TIMER_KWARGS,
        }
        super().__init__(
            interval,
            self.run,
            daemon=True,
            repeats=repeats,
            autostart=autostart,
            repeat_on_exception=repeat_on_exception,
        )

    def run(self):
        return refresh_login(**self.kwargs)

    async def run_async(self):
        return await refresh_login_async(**self.kwargs)


def set_on_demand_token_refresher(token_refresher: OnDemandTokenRefresher, **kwargs):
    key = "OnDemandTokenRefresher"
    sdk = kwargs.get("sdk", SDK)
    if existing := sdk.operation_preprocessors.pop(key, None):
        existing.reset()
    sdk.operation_preprocessors[key] = token_refresher


def unset_on_demand_token_refresher(reset: bool = True, **kwargs):
    key = "OnDemandTokenRefresher"
    sdk = kwargs.get("sdk", SDK)
    if existing := sdk.operation_preprocessors.pop(key, None):
        if not isinstance(existing, OnDemandTokenRefresher):
            raise TypeError(existing)
        if reset:
            existing.reset()
        return existing
    return None


class OnDemandTokenRefresher:
    def __init__(self, refresher: LoginTimerBase, refresh_rate: float = 0.8):
        self._refresher = refresher
        self._refresh_rate = refresh_rate

        self._lock = threading.RLock()
        self._active_refresher: Optional[LoginTimerBase] = None

    def __call__(
        self, operation: Operation, sdk: AccelByteSDK, *args, **kwargs
    ) -> Tuple[Optional[Operation], Optional[HttpResponse]]:
        token_repo = sdk.get_token_repository(raise_when_none=False)
        if token_repo is None:
            return None, HttpResponse.create_token_repo_not_found_error()

        with self._lock:
            needs_refresh = token_repo.has_token() and token_repo.has_token_expired(
                multiplier=self._refresh_rate
            )
            if not needs_refresh:
                return operation, None
            if self._active_refresher is not None:
                if self._active_refresher.status == TimerStatus.RUNNING:
                    return operation, None
                elif self._active_refresher.status == TimerStatus.FINISHED:
                    self._active_refresher = None
                else:
                    raise NotImplementedError()
            self.refresh(token_repo=token_repo)

        return operation, None

    @property
    def status(self) -> TimerStatus:
        if self._active_refresher is None:
            return TimerStatus.IDLE
        return self._active_refresher.status

    def refresh(self, token_repo: TokenRepository):
        if refresh_token := token_repo.get_refresh_token():
            self._active_refresher = RefreshLoginTimer(
                0, refresh_token=refresh_token, autostart=True
            )
        else:
            self._active_refresher = self._refresher.clone(
                interval=0, repeats=None, autostart=True
            )

    def reset(self):
        if self._active_refresher is not None:
            self._active_refresher.cancel()


# endregion repeating



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Group Service."""

__version__ = "2.20.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .get_group_invitation_re_954909 import GetGroupInvitationRequestPublicV1
from .get_group_invite_reques_8cc920 import GetGroupInviteRequestPublicV2
from .get_group_join_request__0b96ac import GetGroupJoinRequestPublicV1
from .get_group_join_request__01154d import GetGroupJoinRequestPublicV2
from .get_my_group_join_request_v2 import GetMyGroupJoinRequestV2



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Qo S Manager Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ModelsListServerResponse
from ...models import ResponseError


class ListServerPerNamespace(Operation):
    """Get list of QoS services per region (ListServerPerNamespace)

    ```
    This endpoint lists all QoS services available in all regions.

    This endpoint is intended to be called by game client to find out all available regions.
    After getting a list of QoS on each region, game client is expected to ping each one with UDP
    connection as described below:

    1. Make UDP connection to each QoS's IP:Port
    2. Send string "PING" after connection established
    3. Wait for string "PONG" response
    4. Note the request-response latency for each QoS in each region

    The game then can use ping latency information to either:
    1. Inform the player on these latencies and let player choose preferred region
    2. Send the latency list to Matchmaking Service so that player can be matched with other players
    in nearby regions
    ```

    Properties:
        url: /qosm/public/namespaces/{namespace}/qos

        method: GET

        tags: ["Public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        status: (status) OPTIONAL str in query

    Responses:
        200: OK - ModelsListServerResponse (list of QoS services returned)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """

    # region fields

    _url: str = "/qosm/public/namespaces/{namespace}/qos"
    _method: str = "GET"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    namespace: str  # REQUIRED in [path]
    status: str  # OPTIONAL in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "status"):
            result["status"] = self.status
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_namespace(self, value: str) -> ListServerPerNamespace:
        self.namespace = value
        return self

    def with_status(self, value: str) -> ListServerPerNamespace:
        self.status = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "status") and self.status:
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, ModelsListServerResponse], Union[None, HttpResponse, ResponseError]
    ]:
        """Parse the given response.

        200: OK - ModelsListServerResponse (list of QoS services returned)

        500: Internal Server Error - ResponseError (Internal Server Error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return ModelsListServerResponse.create_from_dict(content), None
        if code == 500:
            return None, ResponseError.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls, namespace: str, status: Optional[str] = None, **kwargs
    ) -> ListServerPerNamespace:
        instance = cls()
        instance.namespace = namespace
        if status is not None:
            instance.status = status
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ListServerPerNamespace:
        instance = cls()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "status": "status",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "status": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelDeviceBanResponseV4(Model):
    """Model device ban response V4 (model.DeviceBanResponseV4)

    Properties:
        banned_at: (bannedAt) REQUIRED int

        banned_by: (bannedBy) REQUIRED str

        comment: (comment) REQUIRED str

        device_id: (deviceId) REQUIRED str

        device_type: (deviceType) REQUIRED str

        disabled_at: (disabledAt) REQUIRED int

        enabled: (enabled) REQUIRED bool

        end_date: (endDate) REQUIRED int

        id_: (id) REQUIRED str

        namespace: (namespace) REQUIRED str

        reason: (reason) REQUIRED str

        target_namespace: (targetNamespace) REQUIRED str
    """

    # region fields

    banned_at: int  # REQUIRED
    banned_by: str  # REQUIRED
    comment: str  # REQUIRED
    device_id: str  # REQUIRED
    device_type: str  # REQUIRED
    disabled_at: int  # REQUIRED
    enabled: bool  # REQUIRED
    end_date: int  # REQUIRED
    id_: str  # REQUIRED
    namespace: str  # REQUIRED
    reason: str  # REQUIRED
    target_namespace: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_banned_at(self, value: int) -> ModelDeviceBanResponseV4:
        self.banned_at = value
        return self

    def with_banned_by(self, value: str) -> ModelDeviceBanResponseV4:
        self.banned_by = value
        return self

    def with_comment(self, value: str) -> ModelDeviceBanResponseV4:
        self.comment = value
        return self

    def with_device_id(self, value: str) -> ModelDeviceBanResponseV4:
        self.device_id = value
        return self

    def with_device_type(self, value: str) -> ModelDeviceBanResponseV4:
        self.device_type = value
        return self

    def with_disabled_at(self, value: int) -> ModelDeviceBanResponseV4:
        self.disabled_at = value
        return self

    def with_enabled(self, value: bool) -> ModelDeviceBanResponseV4:
        self.enabled = value
        return self

    def with_end_date(self, value: int) -> ModelDeviceBanResponseV4:
        self.end_date = value
        return self

    def with_id(self, value: str) -> ModelDeviceBanResponseV4:
        self.id_ = value
        return self

    def with_namespace(self, value: str) -> ModelDeviceBanResponseV4:
        self.namespace = value
        return self

    def with_reason(self, value: str) -> ModelDeviceBanResponseV4:
        self.reason = value
        return self

    def with_target_namespace(self, value: str) -> ModelDeviceBanResponseV4:
        self.target_namespace = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "banned_at"):
            result["bannedAt"] = int(self.banned_at)
        elif include_empty:
            result["bannedAt"] = 0
        if hasattr(self, "banned_by"):
            result["bannedBy"] = str(self.banned_by)
        elif include_empty:
            result["bannedBy"] = ""
        if hasattr(self, "comment"):
            result["comment"] = str(self.comment)
        elif include_empty:
            result["comment"] = ""
        if hasattr(self, "device_id"):
            result["deviceId"] = str(self.device_id)
        elif include_empty:
            result["deviceId"] = ""
        if hasattr(self, "device_type"):
            result["deviceType"] = str(self.device_type)
        elif include_empty:
            result["deviceType"] = ""
        if hasattr(self, "disabled_at"):
            result["disabledAt"] = int(self.disabled_at)
        elif include_empty:
            result["disabledAt"] = 0
        if hasattr(self, "enabled"):
            result["enabled"] = bool(self.enabled)
        elif include_empty:
            result["enabled"] = False
        if hasattr(self, "end_date"):
            result["endDate"] = int(self.end_date)
        elif include_empty:
            result["endDate"] = 0
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "target_namespace"):
            result["targetNamespace"] = str(self.target_namespace)
        elif include_empty:
            result["targetNamespace"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        banned_at: int,
        banned_by: str,
        comment: str,
        device_id: str,
        device_type: str,
        disabled_at: int,
        enabled: bool,
        end_date: int,
        id_: str,
        namespace: str,
        reason: str,
        target_namespace: str,
        **kwargs,
    ) -> ModelDeviceBanResponseV4:
        instance = cls()
        instance.banned_at = banned_at
        instance.banned_by = banned_by
        instance.comment = comment
        instance.device_id = device_id
        instance.device_type = device_type
        instance.disabled_at = disabled_at
        instance.enabled = enabled
        instance.end_date = end_date
        instance.id_ = id_
        instance.namespace = namespace
        instance.reason = reason
        instance.target_namespace = target_namespace
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelDeviceBanResponseV4:
        instance = cls()
        if not dict_:
            return instance
        if "bannedAt" in dict_ and dict_["bannedAt"] is not None:
            instance.banned_at = int(dict_["bannedAt"])
        elif include_empty:
            instance.banned_at = 0
        if "bannedBy" in dict_ and dict_["bannedBy"] is not None:
            instance.banned_by = str(dict_["bannedBy"])
        elif include_empty:
            instance.banned_by = ""
        if "comment" in dict_ and dict_["comment"] is not None:
            instance.comment = str(dict_["comment"])
        elif include_empty:
            instance.comment = ""
        if "deviceId" in dict_ and dict_["deviceId"] is not None:
            instance.device_id = str(dict_["deviceId"])
        elif include_empty:
            instance.device_id = ""
        if "deviceType" in dict_ and dict_["deviceType"] is not None:
            instance.device_type = str(dict_["deviceType"])
        elif include_empty:
            instance.device_type = ""
        if "disabledAt" in dict_ and dict_["disabledAt"] is not None:
            instance.disabled_at = int(dict_["disabledAt"])
        elif include_empty:
            instance.disabled_at = 0
        if "enabled" in dict_ and dict_["enabled"] is not None:
            instance.enabled = bool(dict_["enabled"])
        elif include_empty:
            instance.enabled = False
        if "endDate" in dict_ and dict_["endDate"] is not None:
            instance.end_date = int(dict_["endDate"])
        elif include_empty:
            instance.end_date = 0
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if "targetNamespace" in dict_ and dict_["targetNamespace"] is not None:
            instance.target_namespace = str(dict_["targetNamespace"])
        elif include_empty:
            instance.target_namespace = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelDeviceBanResponseV4]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelDeviceBanResponseV4]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelDeviceBanResponseV4,
        List[ModelDeviceBanResponseV4],
        Dict[Any, ModelDeviceBanResponseV4],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "bannedAt": "banned_at",
            "bannedBy": "banned_by",
            "comment": "comment",
            "deviceId": "device_id",
            "deviceType": "device_type",
            "disabledAt": "disabled_at",
            "enabled": "enabled",
            "endDate": "end_date",
            "id": "id_",
            "namespace": "namespace",
            "reason": "reason",
            "targetNamespace": "target_namespace",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "bannedAt": True,
            "bannedBy": True,
            "comment": True,
            "deviceId": True,
            "deviceType": True,
            "disabledAt": True,
            "enabled": True,
            "endDate": True,
            "id": True,
            "namespace": True,
            "reason": True,
            "targetNamespace": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Dsm Controller Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsWorkerConfig(Model):
    """Models worker config (models.WorkerConfig)

    Properties:
        auto_deletion: (AutoDeletion) REQUIRED bool

        created_at: (CreatedAt) REQUIRED str

        created_by: (CreatedBy) REQUIRED str

        ghost_worker: (GhostWorker) REQUIRED bool

        manual_buffer_override: (ManualBufferOverride) REQUIRED bool

        modified_by: (ModifiedBy) REQUIRED str

        namespace: (Namespace) REQUIRED str

        updated_at: (UpdatedAt) REQUIRED str

        zombie_worker: (ZombieWorker) REQUIRED bool
    """

    # region fields

    auto_deletion: bool  # REQUIRED
    created_at: str  # REQUIRED
    created_by: str  # REQUIRED
    ghost_worker: bool  # REQUIRED
    manual_buffer_override: bool  # REQUIRED
    modified_by: str  # REQUIRED
    namespace: str  # REQUIRED
    updated_at: str  # REQUIRED
    zombie_worker: bool  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_auto_deletion(self, value: bool) -> ModelsWorkerConfig:
        self.auto_deletion = value
        return self

    def with_created_at(self, value: str) -> ModelsWorkerConfig:
        self.created_at = value
        return self

    def with_created_by(self, value: str) -> ModelsWorkerConfig:
        self.created_by = value
        return self

    def with_ghost_worker(self, value: bool) -> ModelsWorkerConfig:
        self.ghost_worker = value
        return self

    def with_manual_buffer_override(self, value: bool) -> ModelsWorkerConfig:
        self.manual_buffer_override = value
        return self

    def with_modified_by(self, value: str) -> ModelsWorkerConfig:
        self.modified_by = value
        return self

    def with_namespace(self, value: str) -> ModelsWorkerConfig:
        self.namespace = value
        return self

    def with_updated_at(self, value: str) -> ModelsWorkerConfig:
        self.updated_at = value
        return self

    def with_zombie_worker(self, value: bool) -> ModelsWorkerConfig:
        self.zombie_worker = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "auto_deletion"):
            result["AutoDeletion"] = bool(self.auto_deletion)
        elif include_empty:
            result["AutoDeletion"] = False
        if hasattr(self, "created_at"):
            result["CreatedAt"] = str(self.created_at)
        elif include_empty:
            result["CreatedAt"] = ""
        if hasattr(self, "created_by"):
            result["CreatedBy"] = str(self.created_by)
        elif include_empty:
            result["CreatedBy"] = ""
        if hasattr(self, "ghost_worker"):
            result["GhostWorker"] = bool(self.ghost_worker)
        elif include_empty:
            result["GhostWorker"] = False
        if hasattr(self, "manual_buffer_override"):
            result["ManualBufferOverride"] = bool(self.manual_buffer_override)
        elif include_empty:
            result["ManualBufferOverride"] = False
        if hasattr(self, "modified_by"):
            result["ModifiedBy"] = str(self.modified_by)
        elif include_empty:
            result["ModifiedBy"] = ""
        if hasattr(self, "namespace"):
            result["Namespace"] = str(self.namespace)
        elif include_empty:
            result["Namespace"] = ""
        if hasattr(self, "updated_at"):
            result["UpdatedAt"] = str(self.updated_at)
        elif include_empty:
            result["UpdatedAt"] = ""
        if hasattr(self, "zombie_worker"):
            result["ZombieWorker"] = bool(self.zombie_worker)
        elif include_empty:
            result["ZombieWorker"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        auto_deletion: bool,
        created_at: str,
        created_by: str,
        ghost_worker: bool,
        manual_buffer_override: bool,
        modified_by: str,
        namespace: str,
        updated_at: str,
        zombie_worker: bool,
        **kwargs,
    ) -> ModelsWorkerConfig:
        instance = cls()
        instance.auto_deletion = auto_deletion
        instance.created_at = created_at
        instance.created_by = created_by
        instance.ghost_worker = ghost_worker
        instance.manual_buffer_override = manual_buffer_override
        instance.modified_by = modified_by
        instance.namespace = namespace
        instance.updated_at = updated_at
        instance.zombie_worker = zombie_worker
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsWorkerConfig:
        instance = cls()
        if not dict_:
            return instance
        if "AutoDeletion" in dict_ and dict_["AutoDeletion"] is not None:
            instance.auto_deletion = bool(dict_["AutoDeletion"])
        elif include_empty:
            instance.auto_deletion = False
        if "CreatedAt" in dict_ and dict_["CreatedAt"] is not None:
            instance.created_at = str(dict_["CreatedAt"])
        elif include_empty:
            instance.created_at = ""
        if "CreatedBy" in dict_ and dict_["CreatedBy"] is not None:
            instance.created_by = str(dict_["CreatedBy"])
        elif include_empty:
            instance.created_by = ""
        if "GhostWorker" in dict_ and dict_["GhostWorker"] is not None:
            instance.ghost_worker = bool(dict_["GhostWorker"])
        elif include_empty:
            instance.ghost_worker = False
        if (
            "ManualBufferOverride" in dict_
            and dict_["ManualBufferOverride"] is not None
        ):
            instance.manual_buffer_override = bool(dict_["ManualBufferOverride"])
        elif include_empty:
            instance.manual_buffer_override = False
        if "ModifiedBy" in dict_ and dict_["ModifiedBy"] is not None:
            instance.modified_by = str(dict_["ModifiedBy"])
        elif include_empty:
            instance.modified_by = ""
        if "Namespace" in dict_ and dict_["Namespace"] is not None:
            instance.namespace = str(dict_["Namespace"])
        elif include_empty:
            instance.namespace = ""
        if "UpdatedAt" in dict_ and dict_["UpdatedAt"] is not None:
            instance.updated_at = str(dict_["UpdatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "ZombieWorker" in dict_ and dict_["ZombieWorker"] is not None:
            instance.zombie_worker = bool(dict_["ZombieWorker"])
        elif include_empty:
            instance.zombie_worker = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsWorkerConfig]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsWorkerConfig]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsWorkerConfig, List[ModelsWorkerConfig], Dict[Any, ModelsWorkerConfig]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "AutoDeletion": "auto_deletion",
            "CreatedAt": "created_at",
            "CreatedBy": "created_by",
            "GhostWorker": "ghost_worker",
            "ManualBufferOverride": "manual_buffer_override",
            "ModifiedBy": "modified_by",
            "Namespace": "namespace",
            "UpdatedAt": "updated_at",
            "ZombieWorker": "zombie_worker",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "AutoDeletion": True,
            "CreatedAt": True,
            "CreatedBy": True,
            "GhostWorker": True,
            "ManualBufferOverride": True,
            "ModifiedBy": True,
            "Namespace": True,
            "UpdatedAt": True,
            "ZombieWorker": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Inventory Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelsConsumeItem(Model):
    """Apimodels consume item (apimodels.ConsumeItem)

    Properties:
        inventory_id: (inventoryId) REQUIRED str

        qty: (qty) REQUIRED int

        slot_id: (slotId) REQUIRED str

        source_item_id: (sourceItemId) REQUIRED str
    """

    # region fields

    inventory_id: str  # REQUIRED
    qty: int  # REQUIRED
    slot_id: str  # REQUIRED
    source_item_id: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_inventory_id(self, value: str) -> ApimodelsConsumeItem:
        self.inventory_id = value
        return self

    def with_qty(self, value: int) -> ApimodelsConsumeItem:
        self.qty = value
        return self

    def with_slot_id(self, value: str) -> ApimodelsConsumeItem:
        self.slot_id = value
        return self

    def with_source_item_id(self, value: str) -> ApimodelsConsumeItem:
        self.source_item_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "inventory_id"):
            result["inventoryId"] = str(self.inventory_id)
        elif include_empty:
            result["inventoryId"] = ""
        if hasattr(self, "qty"):
            result["qty"] = int(self.qty)
        elif include_empty:
            result["qty"] = 0
        if hasattr(self, "slot_id"):
            result["slotId"] = str(self.slot_id)
        elif include_empty:
            result["slotId"] = ""
        if hasattr(self, "source_item_id"):
            result["sourceItemId"] = str(self.source_item_id)
        elif include_empty:
            result["sourceItemId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls, inventory_id: str, qty: int, slot_id: str, source_item_id: str, **kwargs
    ) -> ApimodelsConsumeItem:
        instance = cls()
        instance.inventory_id = inventory_id
        instance.qty = qty
        instance.slot_id = slot_id
        instance.source_item_id = source_item_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsConsumeItem:
        instance = cls()
        if not dict_:
            return instance
        if "inventoryId" in dict_ and dict_["inventoryId"] is not None:
            instance.inventory_id = str(dict_["inventoryId"])
        elif include_empty:
            instance.inventory_id = ""
        if "qty" in dict_ and dict_["qty"] is not None:
            instance.qty = int(dict_["qty"])
        elif include_empty:
            instance.qty = 0
        if "slotId" in dict_ and dict_["slotId"] is not None:
            instance.slot_id = str(dict_["slotId"])
        elif include_empty:
            instance.slot_id = ""
        if "sourceItemId" in dict_ and dict_["sourceItemId"] is not None:
            instance.source_item_id = str(dict_["sourceItemId"])
        elif include_empty:
            instance.source_item_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsConsumeItem]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsConsumeItem]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsConsumeItem,
        List[ApimodelsConsumeItem],
        Dict[Any, ApimodelsConsumeItem],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "inventoryId": "inventory_id",
            "qty": "qty",
            "slotId": "slot_id",
            "sourceItemId": "source_item_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "inventoryId": True,
            "qty": True,
            "slotId": True,
            "sourceItemId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.import_store_error import ImportStoreError


class ImportFileFormatEnum(StrEnum):
    CSV = "CSV"
    JSON = "JSON"


class ImportStoreHistoryInfo(Model):
    """Import store history info (ImportStoreHistoryInfo)

    Properties:
        created_at: (createdAt) REQUIRED str

        id_: (id) REQUIRED str

        import_file_format: (importFileFormat) REQUIRED Union[str, ImportFileFormatEnum]

        initiated_by: (initiatedBy) REQUIRED str

        namespace: (namespace) REQUIRED str

        store_id: (storeId) REQUIRED str

        success: (success) REQUIRED bool

        errors: (errors) OPTIONAL List[ImportStoreError]

        note: (note) OPTIONAL str
    """

    # region fields

    created_at: str  # REQUIRED
    id_: str  # REQUIRED
    import_file_format: Union[str, ImportFileFormatEnum]  # REQUIRED
    initiated_by: str  # REQUIRED
    namespace: str  # REQUIRED
    store_id: str  # REQUIRED
    success: bool  # REQUIRED
    errors: List[ImportStoreError]  # OPTIONAL
    note: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> ImportStoreHistoryInfo:
        self.created_at = value
        return self

    def with_id(self, value: str) -> ImportStoreHistoryInfo:
        self.id_ = value
        return self

    def with_import_file_format(
        self, value: Union[str, ImportFileFormatEnum]
    ) -> ImportStoreHistoryInfo:
        self.import_file_format = value
        return self

    def with_initiated_by(self, value: str) -> ImportStoreHistoryInfo:
        self.initiated_by = value
        return self

    def with_namespace(self, value: str) -> ImportStoreHistoryInfo:
        self.namespace = value
        return self

    def with_store_id(self, value: str) -> ImportStoreHistoryInfo:
        self.store_id = value
        return self

    def with_success(self, value: bool) -> ImportStoreHistoryInfo:
        self.success = value
        return self

    def with_errors(self, value: List[ImportStoreError]) -> ImportStoreHistoryInfo:
        self.errors = value
        return self

    def with_note(self, value: str) -> ImportStoreHistoryInfo:
        self.note = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "import_file_format"):
            result["importFileFormat"] = str(self.import_file_format)
        elif include_empty:
            result["importFileFormat"] = Union[str, ImportFileFormatEnum]()
        if hasattr(self, "initiated_by"):
            result["initiatedBy"] = str(self.initiated_by)
        elif include_empty:
            result["initiatedBy"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "store_id"):
            result["storeId"] = str(self.store_id)
        elif include_empty:
            result["storeId"] = ""
        if hasattr(self, "success"):
            result["success"] = bool(self.success)
        elif include_empty:
            result["success"] = False
        if hasattr(self, "errors"):
            result["errors"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.errors
            ]
        elif include_empty:
            result["errors"] = []
        if hasattr(self, "note"):
            result["note"] = str(self.note)
        elif include_empty:
            result["note"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        id_: str,
        import_file_format: Union[str, ImportFileFormatEnum],
        initiated_by: str,
        namespace: str,
        store_id: str,
        success: bool,
        errors: Optional[List[ImportStoreError]] = None,
        note: Optional[str] = None,
        **kwargs,
    ) -> ImportStoreHistoryInfo:
        instance = cls()
        instance.created_at = created_at
        instance.id_ = id_
        instance.import_file_format = import_file_format
        instance.initiated_by = initiated_by
        instance.namespace = namespace
        instance.store_id = store_id
        instance.success = success
        if errors is not None:
            instance.errors = errors
        if note is not None:
            instance.note = note
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ImportStoreHistoryInfo:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "importFileFormat" in dict_ and dict_["importFileFormat"] is not None:
            instance.import_file_format = str(dict_["importFileFormat"])
        elif include_empty:
            instance.import_file_format = Union[str, ImportFileFormatEnum]()
        if "initiatedBy" in dict_ and dict_["initiatedBy"] is not None:
            instance.initiated_by = str(dict_["initiatedBy"])
        elif include_empty:
            instance.initiated_by = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "storeId" in dict_ and dict_["storeId"] is not None:
            instance.store_id = str(dict_["storeId"])
        elif include_empty:
            instance.store_id = ""
        if "success" in dict_ and dict_["success"] is not None:
            instance.success = bool(dict_["success"])
        elif include_empty:
            instance.success = False
        if "errors" in dict_ and dict_["errors"] is not None:
            instance.errors = [
                ImportStoreError.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["errors"]
            ]
        elif include_empty:
            instance.errors = []
        if "note" in dict_ and dict_["note"] is not None:
            instance.note = str(dict_["note"])
        elif include_empty:
            instance.note = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ImportStoreHistoryInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ImportStoreHistoryInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ImportStoreHistoryInfo,
        List[ImportStoreHistoryInfo],
        Dict[Any, ImportStoreHistoryInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "id": "id_",
            "importFileFormat": "import_file_format",
            "initiatedBy": "initiated_by",
            "namespace": "namespace",
            "storeId": "store_id",
            "success": "success",
            "errors": "errors",
            "note": "note",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "id": True,
            "importFileFormat": True,
            "initiatedBy": True,
            "namespace": True,
            "storeId": True,
            "success": True,
            "errors": False,
            "note": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "importFileFormat": ["CSV", "JSON"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import ModelsEventResponse
from ..models import ModelsUserLastActivity

from ..operations.user_information import DeleteUserActivitiesHandler
from ..operations.user_information import GetUserActivitiesHandler
from ..operations.user_information import LastUserActivityTimeHandler


@deprecated
@same_doc_as(DeleteUserActivitiesHandler)
def delete_user_activities_handler(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete all user's activities (DeleteUserActivitiesHandler)

    Required permission `NAMESPACE:{namespace}:EVENT [UPDATE]`and scope `analytics`

    Properties:
        url: /event/namespaces/{namespace}/users/{userId}/activities

        method: DELETE

        tags: ["User Information"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - (Unauthorized)

        403: Forbidden - (Forbidden)

        404: Not Found - (Not Found)

        500: Internal Server Error - (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteUserActivitiesHandler.create(
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(DeleteUserActivitiesHandler)
async def delete_user_activities_handler_async(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete all user's activities (DeleteUserActivitiesHandler)

    Required permission `NAMESPACE:{namespace}:EVENT [UPDATE]`and scope `analytics`

    Properties:
        url: /event/namespaces/{namespace}/users/{userId}/activities

        method: DELETE

        tags: ["User Information"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - (Unauthorized)

        403: Forbidden - (Forbidden)

        404: Not Found - (Not Found)

        500: Internal Server Error - (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteUserActivitiesHandler.create(
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetUserActivitiesHandler)
def get_user_activities_handler(
    page_size: int,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get all user's activities (GetUserActivitiesHandler)

    Required permission `NAMESPACE:{namespace}:EVENT [UPDATE]`and scope `analytics`

    Properties:
        url: /event/namespaces/{namespace}/users/{userId}/activities

        method: GET

        tags: ["User Information"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        offset: (offset) OPTIONAL int in query

        page_size: (pageSize) REQUIRED int in query

    Responses:
        200: OK - ModelsEventResponse (OK)

        400: Bad Request - (Bad Request)

        401: Unauthorized - (Unauthorized)

        403: Forbidden - (Forbidden)

        404: Not Found - (Not Found)

        500: Internal Server Error - (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserActivitiesHandler.create(
        page_size=page_size,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetUserActivitiesHandler)
async def get_user_activities_handler_async(
    page_size: int,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get all user's activities (GetUserActivitiesHandler)

    Required permission `NAMESPACE:{namespace}:EVENT [UPDATE]`and scope `analytics`

    Properties:
        url: /event/namespaces/{namespace}/users/{userId}/activities

        method: GET

        tags: ["User Information"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        offset: (offset) OPTIONAL int in query

        page_size: (pageSize) REQUIRED int in query

    Responses:
        200: OK - ModelsEventResponse (OK)

        400: Bad Request - (Bad Request)

        401: Unauthorized - (Unauthorized)

        403: Forbidden - (Forbidden)

        404: Not Found - (Not Found)

        500: Internal Server Error - (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetUserActivitiesHandler.create(
        page_size=page_size,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(LastUserActivityTimeHandler)
def last_user_activity_time_handler(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get last activity time of a user in a namespace (LastUserActivityTimeHandler)

    Required permission `NAMESPACE:{namespace}:EVENT [UPDATE]`and scope `analytics`

    Properties:
        url: /event/namespaces/{namespace}/users/{userId}/lastActivityTime

        method: GET

        tags: ["User Information"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsUserLastActivity (OK)

        401: Unauthorized - (Unauthorized)

        403: Forbidden - (Forbidden)

        404: Not Found - (Not Found)

        500: Internal Server Error - (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = LastUserActivityTimeHandler.create(
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(LastUserActivityTimeHandler)
async def last_user_activity_time_handler_async(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get last activity time of a user in a namespace (LastUserActivityTimeHandler)

    Required permission `NAMESPACE:{namespace}:EVENT [UPDATE]`and scope `analytics`

    Properties:
        url: /event/namespaces/{namespace}/users/{userId}/lastActivityTime

        method: GET

        tags: ["User Information"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsUserLastActivity (OK)

        401: Unauthorized - (Unauthorized)

        403: Forbidden - (Forbidden)

        404: Not Found - (Not Found)

        500: Internal Server Error - (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = LastUserActivityTimeHandler.create(
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Lobby Server."""

__version__ = "3.36.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_join_party_v1 import AdminJoinPartyV1
from .admin_update_party_attr_920eb3 import AdminUpdatePartyAttributesV1
from .public_get_messages import PublicGetMessages



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Ugc Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.models_screenshot_response import ModelsScreenshotResponse


class StatusEnum(StrEnum):
    APPROVED = "APPROVED"
    PENDING = "PENDING"
    REJECTED = "REJECTED"


class ModelsStagingContentResponse(Model):
    """Models staging content response (models.StagingContentResponse)

    Properties:
        channel_id: (channelId) REQUIRED str

        created_time: (createdTime) REQUIRED str

        id_: (id) REQUIRED str

        name: (name) REQUIRED str

        namespace: (namespace) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        tags: (tags) REQUIRED List[str]

        user_id: (userId) REQUIRED str

        custom_attributes: (customAttributes) OPTIONAL Dict[str, Any]

        file_extension: (fileExtension) OPTIONAL str

        note: (note) OPTIONAL str

        payload_url: (payloadURL) OPTIONAL str

        screenshots: (screenshots) OPTIONAL List[ModelsScreenshotResponse]

        sub_type: (subType) OPTIONAL str

        type_: (type) OPTIONAL str

        updated_time: (updatedTime) OPTIONAL str
    """

    # region fields

    channel_id: str  # REQUIRED
    created_time: str  # REQUIRED
    id_: str  # REQUIRED
    name: str  # REQUIRED
    namespace: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    tags: List[str]  # REQUIRED
    user_id: str  # REQUIRED
    custom_attributes: Dict[str, Any]  # OPTIONAL
    file_extension: str  # OPTIONAL
    note: str  # OPTIONAL
    payload_url: str  # OPTIONAL
    screenshots: List[ModelsScreenshotResponse]  # OPTIONAL
    sub_type: str  # OPTIONAL
    type_: str  # OPTIONAL
    updated_time: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_channel_id(self, value: str) -> ModelsStagingContentResponse:
        self.channel_id = value
        return self

    def with_created_time(self, value: str) -> ModelsStagingContentResponse:
        self.created_time = value
        return self

    def with_id(self, value: str) -> ModelsStagingContentResponse:
        self.id_ = value
        return self

    def with_name(self, value: str) -> ModelsStagingContentResponse:
        self.name = value
        return self

    def with_namespace(self, value: str) -> ModelsStagingContentResponse:
        self.namespace = value
        return self

    def with_status(
        self, value: Union[str, StatusEnum]
    ) -> ModelsStagingContentResponse:
        self.status = value
        return self

    def with_tags(self, value: List[str]) -> ModelsStagingContentResponse:
        self.tags = value
        return self

    def with_user_id(self, value: str) -> ModelsStagingContentResponse:
        self.user_id = value
        return self

    def with_custom_attributes(
        self, value: Dict[str, Any]
    ) -> ModelsStagingContentResponse:
        self.custom_attributes = value
        return self

    def with_file_extension(self, value: str) -> ModelsStagingContentResponse:
        self.file_extension = value
        return self

    def with_note(self, value: str) -> ModelsStagingContentResponse:
        self.note = value
        return self

    def with_payload_url(self, value: str) -> ModelsStagingContentResponse:
        self.payload_url = value
        return self

    def with_screenshots(
        self, value: List[ModelsScreenshotResponse]
    ) -> ModelsStagingContentResponse:
        self.screenshots = value
        return self

    def with_sub_type(self, value: str) -> ModelsStagingContentResponse:
        self.sub_type = value
        return self

    def with_type(self, value: str) -> ModelsStagingContentResponse:
        self.type_ = value
        return self

    def with_updated_time(self, value: str) -> ModelsStagingContentResponse:
        self.updated_time = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "channel_id"):
            result["channelId"] = str(self.channel_id)
        elif include_empty:
            result["channelId"] = ""
        if hasattr(self, "created_time"):
            result["createdTime"] = str(self.created_time)
        elif include_empty:
            result["createdTime"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "tags"):
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "custom_attributes"):
            result["customAttributes"] = {
                str(k0): v0 for k0, v0 in self.custom_attributes.items()
            }
        elif include_empty:
            result["customAttributes"] = {}
        if hasattr(self, "file_extension"):
            result["fileExtension"] = str(self.file_extension)
        elif include_empty:
            result["fileExtension"] = ""
        if hasattr(self, "note"):
            result["note"] = str(self.note)
        elif include_empty:
            result["note"] = ""
        if hasattr(self, "payload_url"):
            result["payloadURL"] = str(self.payload_url)
        elif include_empty:
            result["payloadURL"] = ""
        if hasattr(self, "screenshots"):
            result["screenshots"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.screenshots
            ]
        elif include_empty:
            result["screenshots"] = []
        if hasattr(self, "sub_type"):
            result["subType"] = str(self.sub_type)
        elif include_empty:
            result["subType"] = ""
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        if hasattr(self, "updated_time"):
            result["updatedTime"] = str(self.updated_time)
        elif include_empty:
            result["updatedTime"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        channel_id: str,
        created_time: str,
        id_: str,
        name: str,
        namespace: str,
        status: Union[str, StatusEnum],
        tags: List[str],
        user_id: str,
        custom_attributes: Optional[Dict[str, Any]] = None,
        file_extension: Optional[str] = None,
        note: Optional[str] = None,
        payload_url: Optional[str] = None,
        screenshots: Optional[List[ModelsScreenshotResponse]] = None,
        sub_type: Optional[str] = None,
        type_: Optional[str] = None,
        updated_time: Optional[str] = None,
        **kwargs,
    ) -> ModelsStagingContentResponse:
        instance = cls()
        instance.channel_id = channel_id
        instance.created_time = created_time
        instance.id_ = id_
        instance.name = name
        instance.namespace = namespace
        instance.status = status
        instance.tags = tags
        instance.user_id = user_id
        if custom_attributes is not None:
            instance.custom_attributes = custom_attributes
        if file_extension is not None:
            instance.file_extension = file_extension
        if note is not None:
            instance.note = note
        if payload_url is not None:
            instance.payload_url = payload_url
        if screenshots is not None:
            instance.screenshots = screenshots
        if sub_type is not None:
            instance.sub_type = sub_type
        if type_ is not None:
            instance.type_ = type_
        if updated_time is not None:
            instance.updated_time = updated_time
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsStagingContentResponse:
        instance = cls()
        if not dict_:
            return instance
        if "channelId" in dict_ and dict_["channelId"] is not None:
            instance.channel_id = str(dict_["channelId"])
        elif include_empty:
            instance.channel_id = ""
        if "createdTime" in dict_ and dict_["createdTime"] is not None:
            instance.created_time = str(dict_["createdTime"])
        elif include_empty:
            instance.created_time = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "customAttributes" in dict_ and dict_["customAttributes"] is not None:
            instance.custom_attributes = {
                str(k0): v0 for k0, v0 in dict_["customAttributes"].items()
            }
        elif include_empty:
            instance.custom_attributes = {}
        if "fileExtension" in dict_ and dict_["fileExtension"] is not None:
            instance.file_extension = str(dict_["fileExtension"])
        elif include_empty:
            instance.file_extension = ""
        if "note" in dict_ and dict_["note"] is not None:
            instance.note = str(dict_["note"])
        elif include_empty:
            instance.note = ""
        if "payloadURL" in dict_ and dict_["payloadURL"] is not None:
            instance.payload_url = str(dict_["payloadURL"])
        elif include_empty:
            instance.payload_url = ""
        if "screenshots" in dict_ and dict_["screenshots"] is not None:
            instance.screenshots = [
                ModelsScreenshotResponse.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["screenshots"]
            ]
        elif include_empty:
            instance.screenshots = []
        if "subType" in dict_ and dict_["subType"] is not None:
            instance.sub_type = str(dict_["subType"])
        elif include_empty:
            instance.sub_type = ""
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        if "updatedTime" in dict_ and dict_["updatedTime"] is not None:
            instance.updated_time = str(dict_["updatedTime"])
        elif include_empty:
            instance.updated_time = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsStagingContentResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsStagingContentResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsStagingContentResponse,
        List[ModelsStagingContentResponse],
        Dict[Any, ModelsStagingContentResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "channelId": "channel_id",
            "createdTime": "created_time",
            "id": "id_",
            "name": "name",
            "namespace": "namespace",
            "status": "status",
            "tags": "tags",
            "userId": "user_id",
            "customAttributes": "custom_attributes",
            "fileExtension": "file_extension",
            "note": "note",
            "payloadURL": "payload_url",
            "screenshots": "screenshots",
            "subType": "sub_type",
            "type": "type_",
            "updatedTime": "updated_time",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "channelId": True,
            "createdTime": True,
            "id": True,
            "name": True,
            "namespace": True,
            "status": True,
            "tags": True,
            "userId": True,
            "customAttributes": False,
            "fileExtension": False,
            "note": False,
            "payloadURL": False,
            "screenshots": False,
            "subType": False,
            "type": False,
            "updatedTime": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["APPROVED", "PENDING", "REJECTED"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import CreatePolicyVersionRequest
from ..models import CreatePolicyVersionResponse
from ..models import ErrorEntity
from ..models import RetrievePolicyVersionResponse
from ..models import UpdatePolicyVersionRequest
from ..models import UpdatePolicyVersionResponse

from ..operations.policy_versions_with_namespace import CreatePolicyVersion1
from ..operations.policy_versions_with_namespace import PublishPolicyVersion1
from ..operations.policy_versions_with_namespace import RetrieveSinglePolicyVersion1
from ..operations.policy_versions_with_namespace import UpdatePolicyVersion1


@same_doc_as(CreatePolicyVersion1)
def create_policy_version_1(
    policy_id: str,
    body: Optional[CreatePolicyVersionRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a Version from Country-Specific Policy (createPolicyVersion_1)

    Create a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policies/{policyId}/versions

        method: POST

        tags: ["Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreatePolicyVersionRequest in body

        namespace: (namespace) REQUIRED str in path

        policy_id: (policyId) REQUIRED str in path

    Responses:
        201: Created - CreatePolicyVersionResponse (successful operation)

        400: Bad Request - ErrorEntity (40033: errors.net.accelbyte.platform.legal.invalid_policy)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreatePolicyVersion1.create(
        policy_id=policy_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreatePolicyVersion1)
async def create_policy_version_1_async(
    policy_id: str,
    body: Optional[CreatePolicyVersionRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a Version from Country-Specific Policy (createPolicyVersion_1)

    Create a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policies/{policyId}/versions

        method: POST

        tags: ["Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreatePolicyVersionRequest in body

        namespace: (namespace) REQUIRED str in path

        policy_id: (policyId) REQUIRED str in path

    Responses:
        201: Created - CreatePolicyVersionResponse (successful operation)

        400: Bad Request - ErrorEntity (40033: errors.net.accelbyte.platform.legal.invalid_policy)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreatePolicyVersion1.create(
        policy_id=policy_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublishPolicyVersion1)
def publish_policy_version_1(
    policy_version_id: str,
    should_notify: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Manually Publish a Version from Country-Specific Policy (publishPolicyVersion_1)

    Manually publish a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policies/versions/{policyVersionId}/latest

        method: PATCH

        tags: ["Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        policy_version_id: (policyVersionId) REQUIRED str in path

        should_notify: (shouldNotify) OPTIONAL bool in query

    Responses:
        200: OK - (operation successful)

        400: Bad Request - ErrorEntity (40035: errors.net.accelbyte.platform.legal.invalid_policy_version | 40046: errors.net.accelbyte.platform.legal.default_selection_not_provided | 40030: errors.net.accelbyte.platform.legal.localized_policy_version_not_provided)

        409: Conflict - ErrorEntity (40039: errors.net.accelbyte.platform.legal.localized_policy_version_already_latest | 40043: errors.net.accelbyte.platform.legal.localized_policy_version_already_publish)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublishPolicyVersion1.create(
        policy_version_id=policy_version_id,
        should_notify=should_notify,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublishPolicyVersion1)
async def publish_policy_version_1_async(
    policy_version_id: str,
    should_notify: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Manually Publish a Version from Country-Specific Policy (publishPolicyVersion_1)

    Manually publish a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policies/versions/{policyVersionId}/latest

        method: PATCH

        tags: ["Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        policy_version_id: (policyVersionId) REQUIRED str in path

        should_notify: (shouldNotify) OPTIONAL bool in query

    Responses:
        200: OK - (operation successful)

        400: Bad Request - ErrorEntity (40035: errors.net.accelbyte.platform.legal.invalid_policy_version | 40046: errors.net.accelbyte.platform.legal.default_selection_not_provided | 40030: errors.net.accelbyte.platform.legal.localized_policy_version_not_provided)

        409: Conflict - ErrorEntity (40039: errors.net.accelbyte.platform.legal.localized_policy_version_already_latest | 40043: errors.net.accelbyte.platform.legal.localized_policy_version_already_publish)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublishPolicyVersion1.create(
        policy_version_id=policy_version_id,
        should_notify=should_notify,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveSinglePolicyVersion1)
def retrieve_single_policy_version_1(
    policy_id: str,
    version_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Version from Country-Specific Policy (retrieveSinglePolicyVersion_1)

    Retrieve a version of a particular country specific policy. If version is not provided, the Legal Service will assume caller requesting all versions from country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policies/{policyId}/versions

        method: GET

        tags: ["Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        policy_id: (policyId) REQUIRED str in path

        version_id: (versionId) OPTIONAL str in query

    Responses:
        200: OK - List[RetrievePolicyVersionResponse] (successful operation)

        404: Not Found - ErrorEntity (40036: errors.net.accelbyte.platform.legal.policy_version_id_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveSinglePolicyVersion1.create(
        policy_id=policy_id,
        version_id=version_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveSinglePolicyVersion1)
async def retrieve_single_policy_version_1_async(
    policy_id: str,
    version_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Version from Country-Specific Policy (retrieveSinglePolicyVersion_1)

    Retrieve a version of a particular country specific policy. If version is not provided, the Legal Service will assume caller requesting all versions from country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policies/{policyId}/versions

        method: GET

        tags: ["Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        policy_id: (policyId) REQUIRED str in path

        version_id: (versionId) OPTIONAL str in query

    Responses:
        200: OK - List[RetrievePolicyVersionResponse] (successful operation)

        404: Not Found - ErrorEntity (40036: errors.net.accelbyte.platform.legal.policy_version_id_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveSinglePolicyVersion1.create(
        policy_id=policy_id,
        version_id=version_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdatePolicyVersion1)
def update_policy_version_1(
    policy_version_id: str,
    body: Optional[UpdatePolicyVersionRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a Version of Policy (updatePolicyVersion_1)

    Update a particular policy version.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policies/versions/{policyVersionId}

        method: PATCH

        tags: ["Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdatePolicyVersionRequest in body

        namespace: (namespace) REQUIRED str in path

        policy_version_id: (policyVersionId) REQUIRED str in path

    Responses:
        200: OK - UpdatePolicyVersionResponse (successful operation)

        400: Bad Request - ErrorEntity (40035: errors.net.accelbyte.platform.legal.invalid_policy_version | 40042: errors.net.accelbyte.platform.legal.policy_version_freezed)

        409: Conflict - ErrorEntity (40043: errors.net.accelbyte.platform.legal.localized_policy_version_already_exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdatePolicyVersion1.create(
        policy_version_id=policy_version_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdatePolicyVersion1)
async def update_policy_version_1_async(
    policy_version_id: str,
    body: Optional[UpdatePolicyVersionRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a Version of Policy (updatePolicyVersion_1)

    Update a particular policy version.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policies/versions/{policyVersionId}

        method: PATCH

        tags: ["Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdatePolicyVersionRequest in body

        namespace: (namespace) REQUIRED str in path

        policy_version_id: (policyVersionId) REQUIRED str in path

    Responses:
        200: OK - UpdatePolicyVersionResponse (successful operation)

        400: Bad Request - ErrorEntity (40035: errors.net.accelbyte.platform.legal.invalid_policy_version | 40042: errors.net.accelbyte.platform.legal.policy_version_freezed)

        409: Conflict - ErrorEntity (40043: errors.net.accelbyte.platform.legal.localized_policy_version_already_exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdatePolicyVersion1.create(
        policy_version_id=policy_version_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Gdpr Service."""

__version__ = "2.13.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_delete_platform_a_6b01ab import AdminDeletePlatformAccountClosureClient
from .admin_get_platform_acco_4a3f07 import AdminGetPlatformAccountClosureClient
from .admin_update_platform_a_818a23 import AdminUpdatePlatformAccountClosureClient



from typing import Any, Dict, List, Optional, Tuple, Union

import jwt

import accelbyte_py_sdk.services.auth as auth_service
from accelbyte_py_sdk import AccelByteSDK

from ._cache_types import (
    JWKSCache,
    JWTClaims,
    NamespaceContextCache,
    RevocationListCache,
    RolesCache,
)
from ._ctypes import (
    InsufficientPermissionsError,
    TokenRevokedError,
    UserRevokedError,
)
from ._ctypes import (
    PermissionAction,
    PermissionStruct,
    create_permission_struct,
)
from ._utils import (
    replace_resource,
)
from ._validation import validate_permission


class CachingTokenValidator:
    DEFAULT_DECODE_ALGORITHMS: List[str] = ["RS256"]
    DEFAULT_DECODE_OPTIONS: Dict[str, Any] = {"verify_aud": False, "verify_exp": True}
    JWS_HEADER_PARAM_KEY_ID_KEY: str = "kid"

    def __init__(
        self,
        sdk: AccelByteSDK,
        algorithms: Optional[List[str]] = None,
        options: Optional[Dict[str, Any]] = None,
        publisher_namespace: Optional[str] = None,
        token_refresh_interval: Optional[Union[int, float]] = None,
        jwks_refresh_interval: Optional[Union[int, float]] = None,
        revocation_list_refresh_interval: Union[int, float] = 3600,
        role_cache_time: Union[int, float] = 3600,
        namespace_context_cache_time: Union[int, float] = 3600,
    ) -> None:
        self.algorithms = (
            algorithms
            if algorithms is not None
            else CachingTokenValidator.DEFAULT_DECODE_ALGORITHMS
        )
        self.options = (
            options
            if options is not None
            else CachingTokenValidator.DEFAULT_DECODE_OPTIONS
        )
        self.publisher_namespace = publisher_namespace

        if token_refresh_interval is not None:
            auth_service.LoginClientTimer(
                token_refresh_interval,
                repeats=-1,
                autostart=True,
                repeat_on_exception=True,
                sdk=sdk,
            )

        self.jwks_cache = JWKSCache(sdk, jwks_refresh_interval)
        self.revocation_list_cache = RevocationListCache(
            sdk, revocation_list_refresh_interval
        )
        self.roles_cache = RolesCache(sdk, role_cache_time)
        self.namespace_context_cache = NamespaceContextCache(
            sdk, namespace_context_cache_time
        )

        self.jwks_cache.update()
        self.revocation_list_cache.update()
        self.roles_cache.update()
        self.namespace_context_cache.update()

    def has_valid_permissions(
        self,
        claims: JWTClaims,
        permission: PermissionStruct,
        namespace: Optional[str] = None,
        user_id: Optional[str] = None,
    ) -> bool:
        claims_namespace = claims.get("namespace", None)
        user_id = claims.get("user_id", user_id)

        target = create_permission_struct(
            action=permission.action,
            resource=replace_resource(
                resource=permission.resource,
                namespace=namespace,
                token_namespace=claims_namespace,
                publisher_namespace=self.publisher_namespace,
                user_id=user_id,
            ),
        )

        # Check claims.permissions
        claims_permissions = [
            create_permission_struct(action=p.get("Action"), resource=p.get("Resource"))
            for p in claims.get("permissions", [])
        ]
        if claims_permissions and validate_permission(
            target=target,
            permissions=claims_permissions,
            namespace_context_cache=self.namespace_context_cache,
        ):
            return True

        # Check claim.namespace_roles
        claims_namespace_roles = claims.get("namespace_roles", [])
        if user_id and claims_namespace_roles:
            nr_permissions = []
            for nr in claims_namespace_roles:
                nr_permissions.extend(
                    self.roles_cache.get_modified_role_permissions2(
                        namespace_role=nr, user_id=user_id
                    )
                )
            if nr_permissions and validate_permission(
                target=target,
                permissions=nr_permissions,
                namespace_context_cache=self.namespace_context_cache,
            ):
                return True

        # Check claim.roles
        claims_roles = claims.get("roles", [])
        if claims_roles:
            r_permissions = []
            for r in claims_roles:
                r_permissions.extend(
                    self.roles_cache.get_modified_role_permissions(
                        role_id=r, namespace=namespace, user_id=user_id
                    )
                )
            if r_permissions and validate_permission(
                target=target,
                permissions=r_permissions,
                namespace_context_cache=self.namespace_context_cache,
            ):
                return True

        return False

    def validate_token(
        self,
        token: str,
        resource: Optional[str] = None,
        action: Optional[PermissionAction] = None,
        namespace: Optional[str] = None,
        user_id: Optional[str] = None,
        **kwargs,
    ) -> Optional[Exception]:
        # Check if token was revoked.
        if self.revocation_list_cache.is_token_revoked(token=token):
            return TokenRevokedError()

        claims, error = self._decode(token=token, **kwargs)
        if error:
            return error

        # Check if user was revoked.
        if claims_user_id := claims.get("user_id", user_id):
            if self.revocation_list_cache.is_user_revoked(
                user_id=claims_user_id, issued_at=claims.get("iat")
            ):
                return UserRevokedError()

        # Check if the claims has valid permissions.
        if (
            resource is not None
            and action is not None
            and not self.has_valid_permissions(
                claims=claims,
                permission=create_permission_struct(action, resource),
                namespace=namespace,
                user_id=user_id,
            )
        ):
            return InsufficientPermissionsError()

        return None

    def _decode(
        self,
        token: str,
        algorithms: Optional[List[str]] = None,
        options: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> Tuple[Optional[JWTClaims], Optional[Exception]]:
        algorithms = algorithms if algorithms is not None else self.algorithms
        options = options if options is not None else self.options

        header_params = jwt.get_unverified_header(jwt=token)
        if not (
            kid := header_params.get(CachingTokenValidator.JWS_HEADER_PARAM_KEY_ID_KEY)
        ):
            return None, KeyError(CachingTokenValidator.JWS_HEADER_PARAM_KEY_ID_KEY)

        if not (key := self.jwks_cache.get_key(kid)):
            return None, KeyError(kid)

        claims = jwt.decode(
            jwt=token,
            key=key,
            algorithms=algorithms,
            options=options,
        )

        if "user_id" not in claims and (sub := claims.get("sub")):
            claims["user_id"] = sub

        return claims, None


__all__ = [
    "CachingTokenValidator",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.localization import Localization


class ViewCreate(Model):
    """A DTO object for creating view API call. (ViewCreate)

    Properties:
        display_order: (displayOrder) REQUIRED int

        localizations: (localizations) REQUIRED Dict[str, Localization]

        name: (name) REQUIRED str
    """

    # region fields

    display_order: int  # REQUIRED
    localizations: Dict[str, Localization]  # REQUIRED
    name: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_display_order(self, value: int) -> ViewCreate:
        self.display_order = value
        return self

    def with_localizations(self, value: Dict[str, Localization]) -> ViewCreate:
        self.localizations = value
        return self

    def with_name(self, value: str) -> ViewCreate:
        self.name = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "display_order"):
            result["displayOrder"] = int(self.display_order)
        elif include_empty:
            result["displayOrder"] = 0
        if hasattr(self, "localizations"):
            result["localizations"] = {
                str(k0): v0.to_dict(include_empty=include_empty)
                for k0, v0 in self.localizations.items()
            }
        elif include_empty:
            result["localizations"] = {}
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        display_order: int,
        localizations: Dict[str, Localization],
        name: str,
        **kwargs,
    ) -> ViewCreate:
        instance = cls()
        instance.display_order = display_order
        instance.localizations = localizations
        instance.name = name
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> ViewCreate:
        instance = cls()
        if not dict_:
            return instance
        if "displayOrder" in dict_ and dict_["displayOrder"] is not None:
            instance.display_order = int(dict_["displayOrder"])
        elif include_empty:
            instance.display_order = 0
        if "localizations" in dict_ and dict_["localizations"] is not None:
            instance.localizations = {
                str(k0): Localization.create_from_dict(v0, include_empty=include_empty)
                for k0, v0 in dict_["localizations"].items()
            }
        elif include_empty:
            instance.localizations = {}
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ViewCreate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ViewCreate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ViewCreate, List[ViewCreate], Dict[Any, ViewCreate]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "displayOrder": "display_order",
            "localizations": "localizations",
            "name": "name",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "displayOrder": True,
            "localizations": True,
            "name": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Session Browser Service."""

__version__ = "1.18.8"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .add_player_to_session import AddPlayerToSession
from .admin_delete_session import AdminDeleteSession
from .admin_get_session import AdminGetSession
from .admin_query_session import AdminQuerySession
from .admin_search_sessions_v2 import AdminSearchSessionsV2
from .create_session import CreateSession
from .delete_session import DeleteSession
from .delete_session_local_ds import DeleteSessionLocalDS
from .get_active_custom_game__cd6755 import GetActiveCustomGameSessions
from .get_active_matchmaking__0b8050 import GetActiveMatchmakingGameSessions
from .get_recent_player import GetRecentPlayer
from .get_session import GetSession
from .get_session_by_user_i_ds import GetSessionByUserIDs
from .get_session_history_detailed import GetSessionHistoryDetailed
from .get_total_active_session import GetTotalActiveSession
from .join_session import JoinSession
from .remove_player_from_session import RemovePlayerFromSession
from .update_session import UpdateSession
from .update_settings import UpdateSettings
from .user_query_session import UserQuerySession



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Chat Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class ScopeEnum(StrEnum):
    NAMESPACE = "NAMESPACE"
    USER = "USER"


class StatusEnum(StrEnum):
    DRAFT = "DRAFT"
    SENT = "SENT"


class ModelsSaveInboxMessageRequest(Model):
    """Models save inbox message request (models.SaveInboxMessageRequest)

    Properties:
        expired_at: (expiredAt) REQUIRED int

        message: (message) REQUIRED Dict[str, Any]

        scope: (scope) REQUIRED Union[str, ScopeEnum]

        status: (status) REQUIRED Union[str, StatusEnum]

        user_ids: (userIds) REQUIRED List[str]

        category: (category) OPTIONAL str
    """

    # region fields

    expired_at: int  # REQUIRED
    message: Dict[str, Any]  # REQUIRED
    scope: Union[str, ScopeEnum]  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    user_ids: List[str]  # REQUIRED
    category: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_expired_at(self, value: int) -> ModelsSaveInboxMessageRequest:
        self.expired_at = value
        return self

    def with_message(self, value: Dict[str, Any]) -> ModelsSaveInboxMessageRequest:
        self.message = value
        return self

    def with_scope(self, value: Union[str, ScopeEnum]) -> ModelsSaveInboxMessageRequest:
        self.scope = value
        return self

    def with_status(
        self, value: Union[str, StatusEnum]
    ) -> ModelsSaveInboxMessageRequest:
        self.status = value
        return self

    def with_user_ids(self, value: List[str]) -> ModelsSaveInboxMessageRequest:
        self.user_ids = value
        return self

    def with_category(self, value: str) -> ModelsSaveInboxMessageRequest:
        self.category = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "expired_at"):
            result["expiredAt"] = int(self.expired_at)
        elif include_empty:
            result["expiredAt"] = 0
        if hasattr(self, "message"):
            result["message"] = {str(k0): v0 for k0, v0 in self.message.items()}
        elif include_empty:
            result["message"] = {}
        if hasattr(self, "scope"):
            result["scope"] = str(self.scope)
        elif include_empty:
            result["scope"] = Union[str, ScopeEnum]()
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "user_ids"):
            result["userIds"] = [str(i0) for i0 in self.user_ids]
        elif include_empty:
            result["userIds"] = []
        if hasattr(self, "category"):
            result["category"] = str(self.category)
        elif include_empty:
            result["category"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        expired_at: int,
        message: Dict[str, Any],
        scope: Union[str, ScopeEnum],
        status: Union[str, StatusEnum],
        user_ids: List[str],
        category: Optional[str] = None,
        **kwargs,
    ) -> ModelsSaveInboxMessageRequest:
        instance = cls()
        instance.expired_at = expired_at
        instance.message = message
        instance.scope = scope
        instance.status = status
        instance.user_ids = user_ids
        if category is not None:
            instance.category = category
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsSaveInboxMessageRequest:
        instance = cls()
        if not dict_:
            return instance
        if "expiredAt" in dict_ and dict_["expiredAt"] is not None:
            instance.expired_at = int(dict_["expiredAt"])
        elif include_empty:
            instance.expired_at = 0
        if "message" in dict_ and dict_["message"] is not None:
            instance.message = {str(k0): v0 for k0, v0 in dict_["message"].items()}
        elif include_empty:
            instance.message = {}
        if "scope" in dict_ and dict_["scope"] is not None:
            instance.scope = str(dict_["scope"])
        elif include_empty:
            instance.scope = Union[str, ScopeEnum]()
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "userIds" in dict_ and dict_["userIds"] is not None:
            instance.user_ids = [str(i0) for i0 in dict_["userIds"]]
        elif include_empty:
            instance.user_ids = []
        if "category" in dict_ and dict_["category"] is not None:
            instance.category = str(dict_["category"])
        elif include_empty:
            instance.category = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsSaveInboxMessageRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsSaveInboxMessageRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsSaveInboxMessageRequest,
        List[ModelsSaveInboxMessageRequest],
        Dict[Any, ModelsSaveInboxMessageRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "expiredAt": "expired_at",
            "message": "message",
            "scope": "scope",
            "status": "status",
            "userIds": "user_ids",
            "category": "category",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "expiredAt": True,
            "message": True,
            "scope": True,
            "status": True,
            "userIds": True,
            "category": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "scope": ["NAMESPACE", "USER"],
            "status": ["DRAFT", "SENT"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApimodelsCreateInventoryConfigurationReq
from ..models import ApimodelsErrorResponse
from ..models import ApimodelsInventoryConfigurationReq
from ..models import ApimodelsInventoryConfigurationResp
from ..models import ApimodelsListInventoryConfigurationsResp

from ..operations.admin_inventory_configurations import (
    AdminCreateInventoryConfiguration,
)
from ..operations.admin_inventory_configurations import (
    AdminDeleteInventoryConfiguration,
)
from ..operations.admin_inventory_configurations import AdminGetInventoryConfiguration
from ..operations.admin_inventory_configurations import AdminListInventoryConfigurations
from ..operations.admin_inventory_configurations import (
    AdminListInventoryConfigurationsSortByEnum,
)
from ..operations.admin_inventory_configurations import (
    AdminUpdateInventoryConfiguration,
)
from ..models import ApimodelsInventoryConfigurationRespStatusEnum


@same_doc_as(AdminCreateInventoryConfiguration)
def admin_create_inventory_configuration(
    body: ApimodelsCreateInventoryConfigurationReq,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To create inventory configuration (AdminCreateInventoryConfiguration)


    Creating inventory configuration.
    There cannot be one inventory configuration duplicate code per namespace.

    Permission: ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [CREATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations

        method: POST

        tags: ["Admin Inventory Configurations"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsCreateInventoryConfigurationReq in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApimodelsInventoryConfigurationResp (Created)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        409: Conflict - ApimodelsErrorResponse (Conflict)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminCreateInventoryConfiguration.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateInventoryConfiguration)
async def admin_create_inventory_configuration_async(
    body: ApimodelsCreateInventoryConfigurationReq,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To create inventory configuration (AdminCreateInventoryConfiguration)


    Creating inventory configuration.
    There cannot be one inventory configuration duplicate code per namespace.

    Permission: ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [CREATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations

        method: POST

        tags: ["Admin Inventory Configurations"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsCreateInventoryConfigurationReq in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApimodelsInventoryConfigurationResp (Created)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        409: Conflict - ApimodelsErrorResponse (Conflict)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminCreateInventoryConfiguration.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminDeleteInventoryConfiguration)
def admin_delete_inventory_configuration(
    inventory_configuration_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To delete inventory configuration (AdminDeleteInventoryConfiguration)


    Deleting an inventory configuration.
    If an inventory already reference this type (i.e. STATUS is "TIED"), then the type cannot be deleted anymore.

    ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [DELETE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations/{inventoryConfigurationId}

        method: DELETE

        tags: ["Admin Inventory Configurations"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        inventory_configuration_id: (inventoryConfigurationId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminDeleteInventoryConfiguration.create(
        inventory_configuration_id=inventory_configuration_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteInventoryConfiguration)
async def admin_delete_inventory_configuration_async(
    inventory_configuration_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To delete inventory configuration (AdminDeleteInventoryConfiguration)


    Deleting an inventory configuration.
    If an inventory already reference this type (i.e. STATUS is "TIED"), then the type cannot be deleted anymore.

    ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [DELETE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations/{inventoryConfigurationId}

        method: DELETE

        tags: ["Admin Inventory Configurations"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        inventory_configuration_id: (inventoryConfigurationId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminDeleteInventoryConfiguration.create(
        inventory_configuration_id=inventory_configuration_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetInventoryConfiguration)
def admin_get_inventory_configuration(
    inventory_configuration_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To get inventory configuration (AdminGetInventoryConfiguration)


    Getting an inventory configuration info.

    Permission: ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [READ]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations/{inventoryConfigurationId}

        method: GET

        tags: ["Admin Inventory Configurations"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        inventory_configuration_id: (inventoryConfigurationId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsInventoryConfigurationResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetInventoryConfiguration.create(
        inventory_configuration_id=inventory_configuration_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetInventoryConfiguration)
async def admin_get_inventory_configuration_async(
    inventory_configuration_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To get inventory configuration (AdminGetInventoryConfiguration)


    Getting an inventory configuration info.

    Permission: ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [READ]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations/{inventoryConfigurationId}

        method: GET

        tags: ["Admin Inventory Configurations"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        inventory_configuration_id: (inventoryConfigurationId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsInventoryConfigurationResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetInventoryConfiguration.create(
        inventory_configuration_id=inventory_configuration_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminListInventoryConfigurations)
def admin_list_inventory_configurations(
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, AdminListInventoryConfigurationsSortByEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To list inventory configurations (AdminListInventoryConfigurations)


    Listing all inventory configurations in a namespace.
    The response body will be in the form of standard pagination.

    Permission: ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [READ]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations

        method: GET

        tags: ["Admin Inventory Configurations"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

    Responses:
        200: OK - ApimodelsListInventoryConfigurationsResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminListInventoryConfigurations.create(
        code=code,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminListInventoryConfigurations)
async def admin_list_inventory_configurations_async(
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, AdminListInventoryConfigurationsSortByEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To list inventory configurations (AdminListInventoryConfigurations)


    Listing all inventory configurations in a namespace.
    The response body will be in the form of standard pagination.

    Permission: ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [READ]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations

        method: GET

        tags: ["Admin Inventory Configurations"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

    Responses:
        200: OK - ApimodelsListInventoryConfigurationsResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminListInventoryConfigurations.create(
        code=code,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateInventoryConfiguration)
def admin_update_inventory_configuration(
    body: ApimodelsInventoryConfigurationReq,
    inventory_configuration_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To update inventory configuration (AdminUpdateInventoryConfiguration)


    Updating inventory configuration.
    There cannot be duplicate code per namespace.

    Permission: ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [UPDATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations/{inventoryConfigurationId}

        method: PUT

        tags: ["Admin Inventory Configurations"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsInventoryConfigurationReq in body

        inventory_configuration_id: (inventoryConfigurationId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsInventoryConfigurationResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        409: Conflict - ApimodelsErrorResponse (Conflict)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateInventoryConfiguration.create(
        body=body,
        inventory_configuration_id=inventory_configuration_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateInventoryConfiguration)
async def admin_update_inventory_configuration_async(
    body: ApimodelsInventoryConfigurationReq,
    inventory_configuration_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To update inventory configuration (AdminUpdateInventoryConfiguration)


    Updating inventory configuration.
    There cannot be duplicate code per namespace.

    Permission: ADMIN:NAMESPACE:{namespace}:INVENTORY:CONFIGURATION [UPDATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventoryConfigurations/{inventoryConfigurationId}

        method: PUT

        tags: ["Admin Inventory Configurations"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsInventoryConfigurationReq in body

        inventory_configuration_id: (inventoryConfigurationId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsInventoryConfigurationResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        409: Conflict - ApimodelsErrorResponse (Conflict)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateInventoryConfiguration.create(
        body=body,
        inventory_configuration_id=inventory_configuration_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.xbox_achievement import XboxAchievement


class XblAchievementUpdateRequest(Model):
    """A DTO object for update xbox achievement complete percentage API (XblAchievementUpdateRequest)

    Properties:
        achievements: (achievements) OPTIONAL List[XboxAchievement]

        service_config_id: (serviceConfigId) OPTIONAL str

        title_id: (titleId) OPTIONAL str

        xbox_user_id: (xboxUserId) OPTIONAL str
    """

    # region fields

    achievements: List[XboxAchievement]  # OPTIONAL
    service_config_id: str  # OPTIONAL
    title_id: str  # OPTIONAL
    xbox_user_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_achievements(
        self, value: List[XboxAchievement]
    ) -> XblAchievementUpdateRequest:
        self.achievements = value
        return self

    def with_service_config_id(self, value: str) -> XblAchievementUpdateRequest:
        self.service_config_id = value
        return self

    def with_title_id(self, value: str) -> XblAchievementUpdateRequest:
        self.title_id = value
        return self

    def with_xbox_user_id(self, value: str) -> XblAchievementUpdateRequest:
        self.xbox_user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "achievements"):
            result["achievements"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.achievements
            ]
        elif include_empty:
            result["achievements"] = []
        if hasattr(self, "service_config_id"):
            result["serviceConfigId"] = str(self.service_config_id)
        elif include_empty:
            result["serviceConfigId"] = ""
        if hasattr(self, "title_id"):
            result["titleId"] = str(self.title_id)
        elif include_empty:
            result["titleId"] = ""
        if hasattr(self, "xbox_user_id"):
            result["xboxUserId"] = str(self.xbox_user_id)
        elif include_empty:
            result["xboxUserId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        achievements: Optional[List[XboxAchievement]] = None,
        service_config_id: Optional[str] = None,
        title_id: Optional[str] = None,
        xbox_user_id: Optional[str] = None,
        **kwargs,
    ) -> XblAchievementUpdateRequest:
        instance = cls()
        if achievements is not None:
            instance.achievements = achievements
        if service_config_id is not None:
            instance.service_config_id = service_config_id
        if title_id is not None:
            instance.title_id = title_id
        if xbox_user_id is not None:
            instance.xbox_user_id = xbox_user_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> XblAchievementUpdateRequest:
        instance = cls()
        if not dict_:
            return instance
        if "achievements" in dict_ and dict_["achievements"] is not None:
            instance.achievements = [
                XboxAchievement.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["achievements"]
            ]
        elif include_empty:
            instance.achievements = []
        if "serviceConfigId" in dict_ and dict_["serviceConfigId"] is not None:
            instance.service_config_id = str(dict_["serviceConfigId"])
        elif include_empty:
            instance.service_config_id = ""
        if "titleId" in dict_ and dict_["titleId"] is not None:
            instance.title_id = str(dict_["titleId"])
        elif include_empty:
            instance.title_id = ""
        if "xboxUserId" in dict_ and dict_["xboxUserId"] is not None:
            instance.xbox_user_id = str(dict_["xboxUserId"])
        elif include_empty:
            instance.xbox_user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, XblAchievementUpdateRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[XblAchievementUpdateRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        XblAchievementUpdateRequest,
        List[XblAchievementUpdateRequest],
        Dict[Any, XblAchievementUpdateRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "achievements": "achievements",
            "serviceConfigId": "service_config_id",
            "titleId": "title_id",
            "xboxUserId": "xbox_user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "achievements": False,
            "serviceConfigId": False,
            "titleId": False,
            "xboxUserId": False,
        }

    # endregion static methods
